# Apica Grafana Data Source Plugin

<p align="center">
  <img src="https://www.apica.io/wp-content/themes/apica/images/apica_logo.png" alt="drawing" width="338" height="96"/>

</p>

The Apica Grafana Data Source Plugin enables integrating Apica with Grafana to proactively monitors your websites around the clock, along with applications and APIs to identify availability and performance issues before they impact customers.
https://www.apica.io/

![Latest Version Number](https://img.shields.io/badge/Version-2.0.10-orange)

- [Apica Grafana Data Source Plugin](#apica-grafana-data-source-plugin)
- [Supported Grafana Versions](#supported-grafana-versions)
- [Installation and Configuration Instructions](#installation-and-configuration-instructions)
  - [Step 1: Download and install the plugin](#step-1-download-and-install-the-plugin)
    - [Download and unzip the plugin under your grafana plugins directory](#download-and-unzip-the-plugin-under-your-grafana-plugins-directory)
    - [Setup Plugin](#setup-plugin)
    - [Allow Apica unsigned plugins](#allow-apica-unsigned-plugins)
    - [Restart Grafana](#restart-grafana)
  - [Step 2: Add Datasource to Grafana](#step-2-add-datasource-to-grafana)
  - [Step 3: Import Grafana Dashboard Templates](#step-3-import-grafana-dashboard-templates)
- [Dashboards](#dashboards)
  - [Apica Tags SLA](#apica-tags-sla)
  - [Apica Drill Down Checks Aggregate with Tags](#apica-drill-down-checks-aggregate-with-tags)
  - [Apica Check Summary](#apica-check-summary)
  - [Apica Tags Geo Map](#apica-tags-geo-map)
  - [Apica Check Summary Status with Tags](#apica-check-summary-status-with-tags)
  - [Apica Tags Events Insight](#apica-tags-events-insight)
  - [Apica Groups SLA](#apica-groups-sla)
  - [Apica GeoMap Groups](#apica-geomap-groups)
  - [Apica Account Events Insight](#apica-account-events-insight)
  - [Apica Checks Summary Table](#apica-checks-summary-table)
  - [Sample Business Dashboard](#sample-business-dashboard)
- [Template Variables](#template-variables)
- [Ad hoc filter](#ad-hoc-filter)
- [Query Editor Options](#query-editor-options)
  - [Aggregate](#aggregate)
  - [Check Summary](#check-summary)
  - [Metrics](#metrics)
  - [GeoMap](#geomap)
    - [Grafana GeoMap Settings](#grafana-geomap-settings)
  - [Events](#events)
- [Annotations](#annotations)
- [How to Build and Configure Dashboards](#how-to-build-and-configure-dashboards)
  - [Example: Dynamic Dashboard with template variables and ad hoc filter](#example-dynamic-dashboard-with-template-variables-and-ad-hoc-filter)
  - [Example: Custom Dashboard](#example-custom-dashboard)

# Supported Grafana Versions

- Grafana 8.x
- Grafana 9.x
- Grafana 10.x

# Installation and Configuration Instructions

## Step 1: Download and install the plugin

### Download and unzip the plugin under your grafana plugins directory

```
unzip apica-1.0.0.zip -d YOUR_GRAFANA_PLUGIN_DIR/apica
```

### Setup Plugin

Note: Since this is Apica’s private plugin, it must remain “unsigned” per Grafana’s plugin policy.

Contact Apica support directly with any questions.

### Allow Apica unsigned plugins

Allow Apica unsigned plugins in conf/defaults.ini

```
allow_loading_unsigned_plugins = apica-opt-plugin
```

### Restart Grafana

## Step 2: Add Datasource to Grafana

1. Copy your Authentication Token from https://wpm.apicasystem.com/Tools/ApiSettings
   ![Copy API](static/copy-API.png)
2. Open Grafana Configurations-> Data Source

![grafana datas ource settings](static/grafana-datasource-config.png)

3. Add Apica datasource to grafana
   ![add apucagrafana data source](static/add-apica-datasource.png)
4. Enter the url of your Apica API URL <br/>
   Ex. **https://api-wpm2.apicasystem.com/v3**
5. Enter your Authentication Token for the Apica System you copied in step 1

![Data Source Instructions](static/data-source-instructions.png)

## Step 3: Import Grafana Dashboard Templates

![Import Dashboards](static/import-dashboards.png)

# Dashboards

The apica grafana plugin is shipped with below out of the box dashbaords that are available to support visualizing data based on tags, group, check, or account level:

- Tag Level Dashboards
  - [Apica Tags SLA](#apica-tags-sla)
  - [Apica Drill Down Checks Aggregate with Tags](#apica-drill-down-checks-aggregate-with-tags)
  - [Apica Check Summary](#apica-check-summary)
  - [Apica Tags Geo Map](#apica-tags-geo-map)
  - [Apica Tags Events Insight](#apica-tags-events-insight)
- Group Level Dashboards
  - [Apica Groups SLA](#apica-groups-sla)
  - [Apica GeoMap Groups](#apica-geomap-groups)
- Account Level Dashboards
  - [Apica Account Events Insight](#apica-account-events-insight)
  - [Apica Checks Summary Table](#apica-checks-summary-table)
- [Sample Business Dashboard](#sample-business-dashboard)

## Apica Tags SLA

Dynamic Dashboard to show aggregate SLA by selected tags from the dropdown list.

- Allows the team to customize the dashboard to view tags related to them or to focus on business critical checks.
- Drill down on the impacted tag SLA to view checks details associated with the selected tag
- Show custom events associated with the selected tags
- Drill down into check summary dashboard
- Group by selected tag key
- adhoc filter to filter results using specific tag value

![TAGs SLA](static/tags_sla.gif)

## Apica Drill Down Checks Aggregate with Tags

Dynamic Dashboard to show aggregate SLA for selected checks

- Dashboard can be filtered by tags and checks
- Show custom event associated with selected check
- This dashboard is being used a drilldown dashboard from the [Apica Tags SLA](#apica-tags-sla)
- Drilldown into [Apica Check Summary](#apica-check-summary) by clicking on the Check SLA widget

## Apica Check Summary

Dynamic Dashboard to show health status and key KPIs for selected check(s) from the dropdown list

- Dashboard can be filtered by tags and checks
- Show custom event associated with selected check
- Show Annotation in the charts based on custom events
- Dynamic Drill down into Check Details in the Apica System

<br/>

![Apica Check Summary Dashboard](static/apica_check_summary.png)

<br/>

## Apica Tags Geo Map

View checks health status by geo location for selected Tag

- Drill down into Apica systems for check details
- Summary of check types for the selected tag

<br/>

![Apica Tags Geo Map Dashboard](static/tags_geo_map.png)

<br/>

## Apica Check Summary Status with Tags

View multiple checks summary health status for selected Tag

<br/>

![Apica Tags Geo Map Dashboard](static/ApicaCheckSummaryStatuswithTags.png)

<br/>

## Apica Tags Events Insight

Analyze all events triggered in your Apica system in a selected time for the selected tag

- Filter the data based on a given tag filter
- Identify the source of noise in the events like the checks that is triggering most of the events. This will lead for better tuning and quality monitoring

<br/>

![Apica Tags Events Insight Dashboard](static/apica_tag_events_dashbaord.png)

<br/>

## Apica Groups SLA

Dynamic Dashboard to show group Checks SLA

- Show Aggregated SLA at the Group Level
- Show individual checks Aggregated SLA
- You can exclude specific checks from the SLA calculation

<br/>

![Apica Groups SLA Dashboard](static/groups_sla.png)

<br/>

## Apica GeoMap Groups

Dynamic Dashboard to show group checks based on geo location
<br/>

![Apica GeoMap Groups Dashboard](static/apica_geomap_group.png)

<br/>

## Apica Account Events Insight

Analyze all events triggered in your Apica system in a selected time

- This dashboard is at the account level
- Identify the source of noise in the events like the checks that is triggering most of the events. This will lead for better tuning and quality monitoring

<br/>

![Apica Account Events Insight Dashboard](static/apica_account_events_dashbaord.png)

<br/>

## Apica Checks Summary Table

View accounts checks summary health status in table format at the account level

<br/>

![Apica Checks Summary Table](static/ApicaChecksSummaryTable.png)

<br/>

## Sample Business Dashboard

Sample dashboard to demonstrate business impact by using Apica System

- This is a static dashboard so you will need to adjust and customize for your environment
- This dashboard to be used as a template

<br/>

![Sample Business Dashboard](static/business_dashbaord.png)

<br/>

# Template Variables

The plugin supports template Variables that allows you to create more interactive and dynamic dashboards. Instead of hard-coding things like tag, group, and check in your metric queries, you can use variables in their place. Variables are displayed as dropdown lists at the top of the dashboard. These dropdowns make it easy to change the data being displayed in your dashboard. Variables can be used in titles, descriptions, text panels, and queries. Queries with text that starts with $ are templates
For more information please see the Grafana docs
https://grafana.com/docs/grafana/latest/variables/

| Name            | Description                                                                                                                                        | Namespace    | Query   |
| --------------- | -------------------------------------------------------------------------------------------------------------------------------------------------- | ------------ | ------- |
| tags            | List all tags configured on the Apica System                                                                                                       | tags         |         |
| groupByCategory | This variable is used to list all tag keys which are used in the group by category list                                                            | tagsKeys     |         |
| scopeOption     | This variable is used to list scope options which are ) used in the Aggregate Queries. The return values are: (Hour,Day,Week,Month,Year)           | scopeOptions |         |
| checks          | This variable is used to list apica checks. The checks can be filtered also based on tags by providing the tags variable name in the query section | checks       | $tags   |
| groups          | List all groups configured on the Apica System                                                                                                     | groups       |         |
| groupChecks     | This variable is used to list apica checks for a specified group in the query input                                                                | groupChecks  | $groups |

# Ad hoc filter

Ad hoc filters allow you to add key/value filters that are automatically added to all metric queries that use the specified data source. Unlike other variables, you do not use ad hoc filters in queries. Instead, you use ad hoc filters to write filters for existing queries.
https://grafana.com/docs/grafana/latest/variables/variable-types/add-ad-hoc-filters/

The Apica plugin supports ad hoc filter in the tag level queries as a tag filter. For example, if you need to query all aggregate data from a selected tag where environment = prod. To reference the tag filter ad hoc variable you create in your dashbaord, you have to use the $adhoc key work as the value for the "Filter by Tag Expression" field in the Query Editor

The Ad hoc filter supports Grafana operations (=,!=,=~,~!). Note that (=~,~!) allows you to do regex matching

# Query Editor Options

The plugin supports multiple options to Query from the Apica system as detailed in this section. Screenshot below shows the list of Query Types.
There are common fields in the query editor based on your selection

- Level: A list that allows the user to select at which level the query will run. The options in this list can be [account, tag, group, check] based on the Query Category type
- Group: A list that allows you to select the Apica group that your query will run against
- Tag: A list that allows you to select the Apica tag that your query will run against
- Checks: A list that allows you to select the Apica Check that your query will run against
- Filter by Tag Expression: This is used by the Ad hoc filter and only available at the Tag Level queries only
- Filter Check by: This is used by the Check Level Query to filter the list of checks that you will select from in the Query Editor. You can filter the returned checks list using [Account, Tag, Group]

![Query Categories](static/Query_categories.png)

## Aggregate

- Gets aggregated data and SLA for Check, Group, or Tag
- Query Supported Levels: [Tag, Group, Checks]
- Used Apica API: https://api-wpm2.apicasystem.com/v3/help#href-ChecksAggregated

| Filed Name           | Description                                                                                                                                                                                                                                                   | Options                                                                                                                                                                                                  | Additional Info                |
| -------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | ------------------------------ |
| Timerange Type       | Allows the user ro select the aggreate timerange type                                                                                                                                                                                                         | (1) Grafana Time Range: gets aggregated data and SLA between two dates. <br />(2) Most Recent: gets aggregated data and SLA for the most recent periods in a scope. (ie 3 days back if the scope is day) |                                |
| Scope                | String representation of time aggregation scope                                                                                                                                                                                                               | [Hour, Day, Week, Month , Year]                                                                                                                                                                          |                                |
| GroupBy Tag Category | This option is used if you need to group the aggregate results by a specific key. For example, If you need to Query results by Environment tag value                                                                                                          | List of Tag Keys                                                                                                                                                                                         | Available for Tag Level only   |
| Exclude Checks       | This Query field option is used by the Group Level Query only to allow the use to exclude check(s) from the returned results. For example, if you need to calculate SLA for a given group but you need to exclude specific checks in the calculated Group SLA | List of Check IDs in the selected Group                                                                                                                                                                  | Available for Group Level only |

## Check Summary

- Gets a list of checks with summary info about check(s), current SLA, last result and its status
- Query Supported Levels: [Account, Tag, Group, Checks]
- Used Apica API:
  - Account, Tag, Check: https://api-wpm2.apicasystem.com/v3/Help/Route/GET-checks_enabled_severity
  - Group: /checkssummary/groups/<group_id>

## Metrics

- Gets Timeseries data for check results.
- Query Supported Levels: [Check]
- Used Apica API:
  - Metrics Summary: [https://api-wpm2.apicasystem.com/v3/Help/Route/GET-checks-results](https://api-wpm2.apicasystem.com/v3/Help/Route/GET-checks-results_fromUtc_toUtc_detail_level_fromMillisecondsUtc_toMillisecondsUtc)
  - All Metrics: POST /checks/26743/results/metrics
- Selecting a Tag or Group without specifying a Check will return the returned_value metric for all Checks in the given Tag or Group
  <br/>

| Filed Name         | Description                                                                                       | Options                                                                                                                                                                                                                                                                                   | Additional Info                       |
| ------------------ | ------------------------------------------------------------------------------------------------- | ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | ------------------------------------- |
| Return Result Type | Gives the option to select the type of metrics query to run against the selected check            | (1) All Metrics: the user can select from a drop down list the metrics that need to be queries <br />(2) Metrics Summary: This will query the Metrics Summary API that returns multiple metrics that are available for any kind of check like severity, value, result code, message,..etc |                                       |
| Metric             | multi selection drop down list to select the metric names you need to query and get their results | List of metric names                                                                                                                                                                                                                                                                      | only available for All Metrics option |

## GeoMap

- View checks health status by geo location
- Query Supported Levels: [Account, Tag, Group]
- Used Apica API: /checkssummary/locations

### Grafana GeoMap Settings

You need to make sure to define the GeoMap options in order to render the data from this Query in the Grafana Geomap
Marker Color: You can define the marker color based on the "Highest Severity" Field
Marker Size: You can define the marker size based on the "Impacted Checks Count"

## Events

- Analyze all events triggered in your Apica system for any given time that includes both Custom Events and normal events
- Query Supported Levels: [Account, Tag, Check]
- Used Apica API:
  - All Events: [GET /events](https://api-wpm2.apicasystem.com/v3/help#href-CustomEvents)
  - Custom Events: [GET /custom_events](https://api-wpm2.apicasystem.com/v3/help#href-CustomEvents)

# Annotations

The plugin supports annotations by overlaying events like custom events on top of the graphs. When you hover over an annotation you can get the event description.

For example the screenshot below shows a custom event for a deployment that happened on the highlighted time range as annotation

![Annotations](static/annotations.png)

The annotation show custom events was configured as below

![Annotations Config](static/annotations_config.png)

# How to Build and Configure Dashboards

## Example: Dynamic Dashboard with template variables and ad hoc filter

This video will show an example of how to build a dynamic dashboard like the Tags SLA in a step by step

[Video: Tags SLA Dashboard ](https://www.dropbox.com/s/5zbbt3hpt19brt2/apica_dynamics_dashboards.mp4?dl=0)

## Example: Custom Dashboard

In some scenarios you might need to build a custom dashboard for a specfic checks, tags, or groups. The plugin also supports selecting your data from the Query Editor without relying on template variables. In this video we will build

[Video: Custom Dashboard Template](https://www.dropbox.com/s/v6q1gj6lkvcxwdo/apica_static_dashboard.mp4?dl=0)
